package com.huigou.topsun.sap.suppliers.application.impl;

import com.huigou.cache.DictUtil;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.sap.nonProdApply.application.NonProdApplyApplication;
import com.huigou.topsun.sap.nonProdApply.domain.NonProdApplyBaseInfo;
import com.huigou.topsun.sap.nonProdApply.domain.SapNonProdApplyDept;
import com.huigou.topsun.sap.priceMasterData.domain.SapPriceMasterDataItem;
import com.huigou.topsun.sap.suppliers.application.SuppliersDataApplication;
import com.huigou.topsun.sap.suppliers.domain.SuppliersBank;
import com.huigou.topsun.sap.suppliers.domain.SuppliersBuyer;
import com.huigou.topsun.sap.suppliers.domain.SuppliersInfo;
import com.huigou.topsun.sap.suppliers.domain.SuppliersPay;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersBankQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersBuyerQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersDataQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersPayQueryRequest;
import com.huigou.topsun.sap.suppliers.repository.SuppliersBankRepository;
import com.huigou.topsun.sap.suppliers.repository.SuppliersBuyerRepository;
import com.huigou.topsun.sap.suppliers.repository.SuppliersInfoRepository;
import com.huigou.topsun.sap.suppliers.repository.SuppliersPayRepository;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.ApprovalParameter;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.uasp.bpm.HandleResult;
import com.huigou.util.ClassHelper;
import com.huigou.util.StringUtil;
import lombok.SneakyThrows;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.UUID;

@Service("suppliersDataApplication")
public class SuppliersDataApplicationImpl extends FlowBroker implements SuppliersDataApplication {

    @Autowired
    SuppliersInfoRepository suppliersInfoRepository;
    @Autowired
    SuppliersPayRepository suppliersPayRepository;
    @Autowired
    SuppliersBuyerRepository suppliersBuyerRepository;
    @Autowired
    SuppliersBankRepository suppliersBankRepository;

    @Override
    public Map<String, Object> slicedSuppliersDataQuery(SuppliersDataQueryRequest query){
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSuppliersInfoQuery");
        QueryModel model = this.sqlExecutorDao.getQueryModel(queryDescriptor, query);
        if (StringUtil.isNotBlank(query.getBillCode())){
            model.addCriteria(" and t.bill_code like:billCode");
            model.putLikeParam("billCode", query.getBillCode());
        }
        if (StringUtil.isNotBlank(query.getBillCode())){
            model.addCriteria(" and t.name_Org1 like:nameOrg1");
            model.putLikeParam("nameOrg1", query.getNameOrg1());
        }
        if (StringUtil.isNotBlank(query.getPartner())){
            model.addCriteria(" and t.partner like:partner");
            model.putLikeParam("partner", query.getPartner());
        }
//        model.putDictionary("maintenanceType", DictUtil.getDictionary("maintainType"));
//        model.putDictionary("baseUnit", DictUtil.getDictionary("measuringUnit"));
//        model.putDictionary("itemAttributes", DictUtil.getDictionary("suppliesType"));
//        model.putDictionary("businessUsage", DictUtil.getDictionary("serviceusage"));
        model.putDictionary("status", DictUtil.getDictionary("bizBillStatus"));
        return this.sqlExecutorDao.executeSlicedQuery(model);
    }

    @SneakyThrows
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();

        SuppliersInfo suppliersInfo = getBizEntity(SuppliersInfo.class);
        List<SuppliersPay> suppliersPayList = getBizEntities(SuppliersPay.class, "suppliersPayList");
        List<SuppliersBuyer> suppliersBuyerList = getBizEntities(SuppliersBuyer.class, "suppliersBuyerList");
        List<SuppliersBank> suppliersBankList = getBizEntities(SuppliersBank.class, "suppliersBankList");

        if (suppliersInfo.isNew()) {
            suppliersInfo.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            suppliersInfo = (SuppliersInfo) commonDomainService.loadAndFillinProperties(suppliersInfo);
        }
        suppliersInfo = suppliersInfoRepository.save(suppliersInfo);

        for (SuppliersPay pay:suppliersPayList){
            pay.setSId(suppliersInfo.getId());
            this.suppliersPayRepository.save(pay);
        }
        for (SuppliersBuyer buyer:suppliersBuyerList){
            buyer.setSId(suppliersInfo.getId());
            this.suppliersBuyerRepository.save(buyer);
        }

        for (SuppliersBank bank:suppliersBankList){
          bank.setSId(suppliersInfo.getId());
          this.suppliersBankRepository.save(bank);
        }

        return suppliersInfo.getId();
    }

    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(suppliersInfoRepository.getOne(bizId));
    }

    @Override
    protected void onBeforeComplete(DelegateTask delegateTask) {
        super.onBeforeComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (this.isApplyProcUnit(delegateTask) && this.getApprovalParameter().isAdvanceProcessAction()) {
            SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
            suppliersInfo.setStatusId(BizBillStatus.APPROVING.getId());
            suppliersInfoRepository.save(suppliersInfo);
        }
    }
    //流程已办
    @Override
    public void onAfterComplete(DelegateTask delegateTask) {
        super.onAfterComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
      //  SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);

    }

    @Transactional("hanaTransactionManager")
    @Override
    public void onComplete(DelegateTask delegateTask) {
        super.onComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
        //审批不通过
        ApprovalParameter parameter = getApprovalParameter();
       // if (ObjectUtils.equals(parameter.getHandleResult(), HandleResult.DISAGREE.id)) {   //审批不通过 或者退回
      //  }
        if (this.isApplyProcUnit(delegateTask)) {   //申请后直接提交
        }
    }
    /**
     * 流程实例结束
     *
     * @param delegateExecution
     */
    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
        suppliersInfo.setStatusId(status.getId());
        suppliersInfoRepository.save(suppliersInfo);

    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
        suppliersInfo.setStatusId(BizBillStatus.ABORTED.getId());
        suppliersInfoRepository.save(suppliersInfo);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SuppliersInfo suppliersInfo= suppliersInfoRepository.findOne(bizId);
            suppliersInfo.setStatusId(BizBillStatus.APPLYING.getId());
            suppliersInfoRepository.save(suppliersInfo);
        }
    }

    @Override
    public SuppliersInfo loadSuppliersDataById(String id) {

        SuppliersInfo suppliersInfo=suppliersInfoRepository.findOne(id);
        return suppliersInfo;
    }

    @Override
    public Map<String, Object>  loadSuppliersPayDatas(SuppliersPayQueryRequest queryRequest){
       // System.out.println("sId=============="+sId);
       // Sort sort = new Sort(Sort.Direction.ASC, "sequence");
      //  List<SuppliersPay> list=suppliersPayRepository.findAllBySId(sId, sort);

        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSuppliersPayList");
       // SuppliersPayQueryRequest queryRequest=new  SuppliersPayQueryRequest();
       // queryRequest.setSId(sId);
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
       // queryModel.putDictionary("konwa", DictUtil.getDictionary("currency"));
        queryModel.addCriteria(" and t.s_id =:sId");
        queryModel.putParam("sId", queryRequest.getSId());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }
    @Override
    public Map<String, Object>  loadSuppliersBuyerDatas(SuppliersBuyerQueryRequest queryRequest){
        //return  this.suppliersBuyerRepository.findBysId(sId);
       // Sort sort = new Sort(Sort.Direction.ASC, "sequence");
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSuppliersBuyerList");

        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.addCriteria(" and t.s_id =:sId");
        queryModel.putParam("sId", queryRequest.getSId());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }
    @Override
    public Map<String, Object>  loadSuppliersBankDatas(SuppliersBankQueryRequest  queryRequest){
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSuppliersBankList");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.addCriteria(" and t.s_id =:sId");
        queryModel.putParam("sId", queryRequest.getSId());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    //删除数据
    @Override
    public void   deleteSuppliersDataByIds( List<String> ids){
        for  (String id:ids){
            List<SuppliersPay> suppliersPayList=this.suppliersPayRepository.findAllBySId(id);
            for (SuppliersPay suppliersPay:suppliersPayList){
                suppliersPayRepository.delete(suppliersPay);
            }
            List<SuppliersBuyer> suppliersBuyerList=this.suppliersBuyerRepository.findAllBySId(id);
            for (SuppliersBuyer suppliersBuyer:suppliersBuyerList){
                suppliersBuyerRepository.delete(suppliersBuyer);
            }
            List<SuppliersBank> suppliersBankList=this.suppliersBankRepository.findAllBySId(id);
            for (SuppliersBank suppliersBank:suppliersBankList){
                suppliersBankRepository.delete(suppliersBank);
            }
             this.suppliersInfoRepository.delete(id);
        }
    }
}
