package com.huigou.topsun.sap.suppliers.controller;

import com.huigou.context.Operator;
import com.huigou.context.OrgUnit;
import com.huigou.topsun.base.bsnMessage.domain.BsnMessageReject;
import com.huigou.topsun.sap.nonProdApply.application.NonProdApplyApplication;
import com.huigou.topsun.sap.nonProdApply.domain.NonProdApplyBaseInfo;
import com.huigou.topsun.sap.nonProdApply.domain.query.NonProdApplyBaseInfoQueryRequest;
import com.huigou.topsun.sap.priceMasterData.domain.query.SapPriceMasterDataItemQueryRequest;
import com.huigou.topsun.sap.suppliers.application.SuppliersDataApplication;
import com.huigou.topsun.sap.suppliers.domain.SuppliersBank;
import com.huigou.topsun.sap.suppliers.domain.SuppliersBuyer;
import com.huigou.topsun.sap.suppliers.domain.SuppliersInfo;
import com.huigou.topsun.sap.suppliers.domain.SuppliersPay;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersBankQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersBuyerQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersDataQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersPayQueryRequest;
import com.huigou.uasp.annotation.ControllerMapping;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.client.CommonController;
import com.huigou.util.SDO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import java.util.List;
import java.util.Map;

/**
 * @Auther: LXH
 *
 * @Date: 2024/03/01
 * @Description:
 */
@Controller
@ControllerMapping("/sapSuppliersData")
public class SuppliersDataController extends CommonController {

    protected String getPagePath() {
        return "/biz/topsun/sap/suppliers/";
    }


    @Autowired
    SuppliersDataApplication suppliersDataApplication;


    public String forwardSuppliersDataList() {
        return forward("suppliersDataList");
    }


    public String slicedSuppliersDataList() {
        SDO sdo = this.getSDO();
        SuppliersDataQueryRequest queryRequest = sdo.toQueryRequest(SuppliersDataQueryRequest.class);
        Map<String, Object> map = suppliersDataApplication.slicedSuppliersDataQuery(queryRequest);
        return toResult(map);
    }


    public String forwardSuppliersDataApply() {

       this.putAttribute("processDefinitionKey",SuppliersDataApplication.PROCESS_DEFINITION_KEY);
       this.putAttribute("procUnitId","Apply");//定义流程环节的id

        SuppliersInfo suppliersInfo = new SuppliersInfo();
        suppliersInfo.setStatusId(BizBillStatus.APPLYING.getId());
        Operator operator = getOperator();//获取当前操作员的详情信息
        suppliersInfo.setDefaultValues(new OrgUnit(operator.getFullId(),operator.getFullName()));
        return  forward("suppliersDataDetail",suppliersInfo);
   }

    public String showSuppliersData() {
        SDO sdo = this.getSDO();
        String id = sdo.getBizId();
        String isDetailPage = sdo.getString("isDetailPage");
        if("true".equals(isDetailPage)) this.putAttribute("isDetailPage","true");
        SuppliersInfo suppliersInfo = suppliersDataApplication.loadSuppliersDataById(id);
        return forward("suppliersDataDetail", suppliersInfo);
    }

    public String  slicedSuppliersPayData(){
        SDO sdo = this.getSDO();
        SuppliersPayQueryRequest queryRequest=sdo.toQueryRequest(SuppliersPayQueryRequest.class);
        queryRequest.setSId(sdo.getString("sId"));
        Map<String, Object> map = suppliersDataApplication.loadSuppliersPayDatas(queryRequest);
        return toResult(map);
     }

    public String slicedSuppliersBuyerData(){
        SDO sdo = this.getSDO();
        SuppliersBuyerQueryRequest queryRequest=sdo.toQueryRequest(SuppliersBuyerQueryRequest.class);
        queryRequest.setSId(sdo.getString("sId"));
        Map<String, Object> map = suppliersDataApplication.loadSuppliersBuyerDatas(queryRequest);
        return toResult(map);
    }

    public String  slicedSuppliersBankData(){
        SDO sdo = this.getSDO();
        SuppliersBankQueryRequest queryRequest=sdo.toQueryRequest(SuppliersBankQueryRequest.class);
        queryRequest.setSId(sdo.getString("sId"));
        Map<String, Object> map = suppliersDataApplication.loadSuppliersBankDatas(queryRequest);
        return toResult(map);
    }
    public String deleteSuppliersData() {
        SDO sdo = this.getSDO();
        List<String> ids = sdo.getIds();
        suppliersDataApplication.deleteSuppliersDataByIds(ids);
        return success();
    }
}
